use warnings;
use strict;

print "MESSAGE: This script is to add dummy parents (parents whose information is not listed in the Merlin format PED file, to keep pedigree structure \"complete\" for further analysis in RMW. Running this script is optional.\n";
print "\t--Shuang Feng, April 15, 2014\n\n";
print "USAGE: perl preprocess.merlin.ped.pl [PEDFILE] [DATFILE] [OUTFILE]\n\n";
print "[PEDFILE] is the original MERLIN format PED file.\n";
print "[DATFILE] is the MERLIN format DAT file.\n";
print "[OUTFILE] is the file name of the output PED file.\n";

die "FATAL ERROR: Number of arguments must be 3!\n" if ($#ARGV!=2);
my $pedfile = $ARGV[0];
my $datfile = $ARGV[1];
my $outfile = $ARGV[2];

print "You are using the following arguments:\n";
print "\t [PEDFILE]: $ARGV[0]\n";
print "\t [DATFILE]: $ARGV[1]\n";
print "\t [OUTFILE]: $ARGV[2]\n";

my $cmd;
my @famid;
my @pid;
my @fid;
my @mid;
my @line;
my @label;
my @addFather;
my @addMother;

#check if there is any dummy parents
print `awk '{print \$1"\t"\$2"\t"\$3"\t"\$4"\t"\$5}' $pedfile > tmp.merlin.reformating`;
open (PEDIN, 'tmp.merlin.reformating');
open (OUT,'> added.ped');
while (<PEDIN>) 
{
   chomp;
   @line = split(/\t/,$_);
   push (@famid, $line[0]);
   push (@pid, $line[1]);
   push (@fid, $line[2]);
   push (@mid, $line[3]);
}
close (PEDIN); 

my $tmp = `awk '{print \$1}' $datfile | tr '\n' '\t'`;
@label = split(/\t/,$tmp);

#match parent IDs to pids.

foreach my $i (0..$#fid) 
{
   my $added=0;
   foreach my $add (0..$#addFather)
   {
      if($fid[$i] eq $addFather[$add])
      {
	 $added=1;
	 last;
      }
   }
   if($added==1)
   {
      next;
   }
   my $idx=-1;

   foreach my $j (0..$#pid)
   {
      if($fid[$i] eq $pid[$j])
      {
	 $idx=$j;
	 last;
      }
   }
#if not found, then add this record to ped file
   if($idx==-1 && $fid[$i] != 0)
   {
      push(@addFather,$fid[$i]);
      print OUT "$famid[$i]\t$fid[$i]\t0\t0\t1\t";
      foreach my $l (0..$#label)
      {
	 if($label[$l] eq 'T' || $label[$l] eq 'C')
	 {
	    print OUT "x\t";
	 }
	 elsif($label[$l] eq 'M' || $label[$l] eq 'S2')
	 {
	    print OUT "0\t0\t";
	 }
      }
      print OUT "\n";
   }
}

foreach my $i (0..$#mid) 
{
   my $added=0;
   foreach my $add (0..$#addMother)
   {    
      if($mid[$i] eq $addMother[$add])
      {         
	 $added=1;      
	 last;          
      }         
   }    
   if($added==1)
   {    
      next;     
   }    
   my $idx=-1;
   foreach my $j (0..$#pid)
   {
      if($mid[$i] eq $pid[$j])
      {
	 $idx=$j;
	 last;
      }
   }
#if not found, then add this record to ped file
   if($idx==-1 && $mid[$i]!=0)
   {
      push(@addMother,$mid[$i]);
      print OUT "$famid[$i]\t$mid[$i]\t0\t0\t2\t";
      foreach my $l (0..$#label)
      {
	 if($label[$l] eq 'T' || $label[$l] eq 'C')
	 {
	    print OUT "x\t";
	 }
	 elsif($label[$l] eq 'M' || $label[$l] eq 'S2')
	 {
	    print OUT "0\t0\t";
	 }
      }
      print OUT "\n";
   }
}

close(OUT);

print `cat added.ped $pedfile > $outfile`;
print `rm tmp`;

print "Process completed. Recommend using pedstats to check the integrity of your PED/DAT files.\n";
